//---------------------------------------------------------------------------
// TButton Class
//---------------------------------------------------------------------------

/*
Supported properties:

  Left
  Top
  Width
  Height
  Caption
  Hint
  ShowHint
  Enabled
  Font.Color
  Font.Charset
  Font.Height
  Font.Name
  Font.Style
  Cursor
  ModalResult
  
Supported events:

  OnClick
  OnKeyDown
  OnKeyUp

*/

void DFMParser::TButtonClass ()
{
  // properties
  int  Left;
  int  Top;
  int  Width;
  int  Height;
  char *Caption = NULL;
  char *Hint = NULL;
  char *ShowHint = NULL;
  char *Enabled = NULL;
  char *FontColor = NULL;
  char *FontCharset = NULL;
  int FontHeight = 0;
  char *FontName = NULL;
  StackElement *FontStyle = NULL;
  char *Cursor = NULL;
  int ModalResult;

  // Events
  char *OnClick = NULL;
  char *OnKeyDown = NULL;
  char *OnKeyUp = NULL;

  // read words...
  bool Error;
  char *Name = ReadString (Error);
  while (strcmp (Name,"object") != 0 && strcmp (Name,"end") != 0)
    {
      // Left
      ReadDigitProperty (Name,"Left",Left);

      // Top
      ReadDigitProperty (Name,"Top",Top);

      // Height
      ReadDigitProperty (Name,"Height",Height);

      // Width
      ReadDigitProperty (Name,"Width",Width);

      // Caption
      ReadQuotationProperty (Name,"Caption",Caption);

      // Hint
      ReadQuotationProperty (Name,"Hint",Hint);

      // ShowHint
      ReadStringProperty (Name,"ShowHint",ShowHint);

      // Enabled
      ReadStringProperty (Name,"Enabled",Enabled);

      // Font Charset
      ReadStringProperty (Name,"Font.Charset",FontCharset);

      // Font Height
      ReadDigitProperty (Name,"Font.Height",FontHeight);

      // Font Name
      ReadQuotationProperty (Name,"Font.Name",FontName);

      // Font Color
      ReadStringProperty (Name,"Font.Color",FontColor);

      // Font Style
      ReadSetProperty (Name,"Font.Style",FontStyle);

      // Cursor
      ReadStringProperty (Name,"Cursor",Cursor);

      // OnClick
      ReadStringProperty (Name,"OnClick",OnClick);

      // OnKeyDown
      ReadStringProperty (Name,"OnKeyDown",OnKeyDown);

      // OnKeyUp
      ReadStringProperty (Name,"OnKeyUp",OnKeyUp);

      // ModalResult
      ReadDigitProperty (Name,"ModalResult",ModalResult);

      // read new word ...
      delete [] Name;
      Name = ReadString (Error);
    }
  delete [] Name;

  // *.h file
  fprintf (hfile,"  FXButton *%s;\n",Stack -> Name);

  // *.cpp file
  fprintf (cppfile,"  %s = new FXButton (%s,\"",Stack -> Name,Stack -> Next -> Name);

  // Caption
  if (Caption != NULL)
    {
      fprintf (cppfile,"%s",Caption);
    }

  // Hint
  if (Hint != NULL)
    {
      if (ShowHint == NULL || strcmp (ShowHint,"True") == 0)
        {
          fprintf (cppfile,"\\t%s",Hint);
        }
    }

  // any event - OnClick, OnKeyDown or OnKeyUp
  if (OnClick != NULL || OnKeyDown != NULL || OnKeyUp != NULL)
    {
      fprintf (cppfile,"\",NULL,this,ID_%s",Stack -> Name,OnClick);
    }
  else
    fprintf (cppfile,"\",NULL,NULL,0");

  fprintf (cppfile,",FRAME_NORMAL|LAYOUT_FIX_X|LAYOUT_FIX_Y|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT,");
  fprintf (cppfile,"%i,%i,%i,%i,0,0,0,0);\n",Left,Top,Width,Height);

  // Enabled
  if (Enabled != NULL && strcmp (Enabled,"False") == 0)
    {
      fprintf (cppfile,"  %s -> disable ();\n",Stack -> Name);
    }

  // Font Color

  if (FontColor != NULL && strlen (GetColor (FontColor)) > 0)
    {
      fprintf (cppfile,"  %s -> setTextColor (%s);\n",Stack -> Name,GetColor (FontColor));
    }

  // Font
  if (FontHeight != 0 && FontName != NULL)
    {
      fprintf (cppfile,"  %s -> setFont (new FXFont (getApp(),\"%s\",%i",Stack -> Name,FontName,(int)((-FontHeight * 72) / 96));

      bool Bold = false;
      bool Italic = false;
      while (!Empty (FontStyle))
        {
          if (strcmp (FontStyle -> Name,"fsBold") == 0)
            Bold = true;
          else
          if (strcmp (FontStyle -> Name,"fsItalic") == 0)
            Italic = true;
          Pop (FontStyle);
        }

      // Bold
      if (Bold == true)
        fprintf (cppfile,",FONTWEIGHT_BOLD");
      else
        fprintf (cppfile,",FONTWEIGHT_NORMAL");

      // Italic
      if (Italic == true)
        fprintf (cppfile,",FONTSLANT_ITALIC");
      else
        fprintf (cppfile,",FONTSLANT_REGULAR");

      // FontCharset
      if (FontCharset != NULL)
        fprintf (cppfile,",%s));\n",GetFontCharset (FontCharset));
      else
        fprintf (cppfile,"));\n");
    }

  // Cursor
  if (Cursor != NULL && strlen (GetCursor (Cursor)) > 0)
    {
      fprintf (cppfile,"  %s -> setDefaultCursor(getApp () -> getDefaultCursor(%s));\n",Stack -> Name,GetCursor (Cursor));
    }

  // ModalResult
  if (ModalResult != 0 && ClassGenerate == DIALOG_BOX)
    {
      switch (ModalResult)
        {
          case 1:  // mrOK
            fprintf (cppfile,"  %s -> setTarget(this);\n",Stack -> Name);
            fprintf (cppfile,"  %s -> setSelector(FXDialogBox::ID_ACCEPT);\n",Stack -> Name);
            break;
          case 2:  // mrCancel
            fprintf (cppfile,"  %s -> setTarget(this);\n",Stack -> Name);
            fprintf (cppfile,"  %s -> setSelector(FXDialogBox::ID_CANCEL);\n",Stack -> Name);
            break;
          default:
            break;
        }
    }

  // Event - OnClick
  if (OnClick != NULL)
    {
      // EventFunction
      Push (EventsFunction,OnClick);

      // Message Type
      Push (MessageType,"SEL_COMMAND");

      // MessageID
      char MsgID [512];
      sprintf (MsgID,"ID_%s",Stack -> Name);
      Push (MessageID,MsgID);
    }

  // Event - OnKeyDown
  if (OnKeyDown != NULL)
    {
      // EventFunction
      Push (EventsFunction,OnKeyDown);

      // Message Type
      Push (MessageType,"SEL_KEYPRESS");

      // MessageID
      char MsgID [512];
      sprintf (MsgID,"ID_%s",Stack -> Name);
      Push (MessageID,MsgID);
    }

  // Event - OnKeyUp
  if (OnKeyUp != NULL)
    {
      // EventFunction
      Push (EventsFunction,OnKeyUp);

      // Message Type
      Push (MessageType,"SEL_KEYRELEASE");

      // MessageID
      char MsgID [512];
      sprintf (MsgID,"ID_%s",Stack -> Name);
      Push (MessageID,MsgID);
    }

  // one empty line
  fprintf (cppfile,"\n");

  // clear
  delete [] Caption;
  delete [] FontColor;
  delete [] Hint;
  delete [] ShowHint;
  delete [] FontCharset;
  delete [] FontName;
  delete [] Enabled;
  delete [] Cursor;
  delete [] OnClick;
  delete [] OnKeyDown;
  delete [] OnKeyUp;
}
