//---------------------------------------------------------------------------
// TCheckBox Class
//---------------------------------------------------------------------------

/*
Supported properties:

  Left
  Top
  Width
  Height
  Caption
  Color
  Hint
  ShowHint
  Alignment
  Font.Color
  Font.Charset
  Font.Height
  Font.Name
  Font.Style
  Cursor
  Enabled

Supported events:
  OnClick
*/

void DFMParser::TCheckBoxClass ()
{
  // attributes
  int  Left;
  int  Top;
  int  Width;
  int  Height;
  char *Caption = NULL;
  char *FontColor = NULL;
  char *Color = NULL;
  char *Hint = NULL;
  char *ShowHint = NULL;
  char *Alignment = NULL;
  char *FontCharset = NULL;
  int FontHeight = 0;
  char *FontName = NULL;
  StackElement *FontStyle = NULL;
  char *Cursor = NULL;
  char *Enabled = NULL;
  char *Checked = NULL;

  // Events
  char *OnClick = NULL;

  // read words...
  bool Error;
  char *Name = ReadString (Error);
  while (strcmp (Name,"object") != 0 && strcmp (Name,"end") != 0)
    {
      // Left
      ReadDigitProperty (Name,"Left",Left);

      // Top
      ReadDigitProperty (Name,"Top",Top);

      // Height
      ReadDigitProperty (Name,"Height",Height);

      // Width
      ReadDigitProperty (Name,"Width",Width);

      // Caption
      ReadQuotationProperty (Name,"Caption",Caption);

      // Color
      ReadStringProperty (Name,"Color",Color);

      // Hint
      ReadQuotationProperty (Name,"Hint",Hint);

      // ShowHint
      ReadStringProperty (Name,"ShowHint",ShowHint);

      // Layout
      //ReadStringProperty (Name,"Layout",Layout);

      // Alignment
      ReadStringProperty (Name,"Alignment",Alignment);

      // Font Charset
      ReadStringProperty (Name,"Font.Charset",FontCharset);

      // Font Color
      ReadStringProperty (Name,"Font.Color",FontColor);

      // Font Height
      ReadDigitProperty (Name,"Font.Height",FontHeight);

      // Font Name
      ReadQuotationProperty (Name,"Font.Name",FontName);

      // Font Style
      ReadSetProperty (Name,"Font.Style",FontStyle);

      // Cursor
      ReadStringProperty (Name,"Cursor",Cursor);

      // Enabled
      ReadStringProperty (Name,"Enabled",Enabled);

      // Checked
      ReadStringProperty (Name,"Checked",Checked);

      // OnClick
      ReadStringProperty (Name,"OnClick",OnClick);

      // clear old and read new word ...
      delete [] Name;
      Name = ReadString (Error);
    }
  delete [] Name;

  // *.h file
  fprintf (hfile,"  FXCheckButton *%s;\n",Stack -> Name);

  // *.cpp file
  fprintf (cppfile,"  %s = new FXCheckButton (%s,",Stack -> Name,Stack -> Next -> Name);

  // Caption
  if (Caption != NULL)
    {
      fprintf (cppfile,"\"%s",Caption);
    }

  // Hint
  if (Hint != NULL)
    {
      if (ShowHint == NULL || strcmp (ShowHint,"True") == 0)
        {
          fprintf (cppfile,"\\t%s",Hint);
        }
    }

  // any Event 
  if (OnClick != NULL)
    {
      fprintf (cppfile,"\",this,ID_%s",Stack -> Name,OnClick);
    }
  else
    fprintf (cppfile,"\",NULL,0");

  fprintf (cppfile,",LAYOUT_FIX_X|LAYOUT_FIX_Y|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT");

  // Alignment
  if (Alignment != NULL && strcmp (Alignment,"taLeftJustify") == 0)
    {
      fprintf (cppfile,"|ICON_AFTER_TEXT|LAYOUT_FILL_X");
    }
  else

      fprintf (cppfile,"|ICON_BEFORE_TEXT");

  // Left, Top, Width, Height
  fprintf (cppfile,",%i,%i,%i,%i,0,0,0,0);\n",Left,Top,Width,Height);

  // Checked
  if (Checked != NULL && strcmp (Checked,"True") == 0)
    {
      fprintf (cppfile,"  %s -> setCheck (TRUE);\n",Stack -> Name);
    }

  // Font Color
  if (FontColor != NULL && strlen (GetColor (FontColor)) > 0)
    {
      fprintf (cppfile,"  %s -> setTextColor (%s);\n",Stack -> Name,GetColor (FontColor));
    }

  // Color
  if (Color != NULL && strlen (GetColor (Color)) > 0)
    {
      fprintf (cppfile,"  %s -> setBackColor (%s);\n",Stack -> Name,GetColor (Color));
    }

  // Font
  if (FontHeight != 0 && FontName != NULL)
    {
      fprintf (cppfile,"  %s -> setFont (new FXFont (getApp(),\"%s\",%i",Stack -> Name,FontName,(int)((-FontHeight * 72) / 96));

      bool Bold = false;
      bool Italic = false;
      while (!Empty (FontStyle))
        {
          if (strcmp (FontStyle -> Name,"fsBold") == 0)
            Bold = true;
          else
          if (strcmp (FontStyle -> Name,"fsItalic") == 0)
            Italic = true;
          Pop (FontStyle);
        }

      // Bold
      if (Bold == true)
        fprintf (cppfile,",FONTWEIGHT_BOLD");
      else
        fprintf (cppfile,",FONTWEIGHT_NORMAL");

      // Italic
      if (Italic == true)
        fprintf (cppfile,",FONTSLANT_ITALIC");
      else
        fprintf (cppfile,",FONTSLANT_REGULAR");

      // Charset
      if (FontCharset != NULL)
        fprintf (cppfile,",%s));\n",GetFontCharset (FontCharset));
      else
        fprintf (cppfile,"));\n");
    }

  // Cursor
  if (Cursor != NULL && strlen (GetCursor (Cursor)) > 0)
    {
      fprintf (cppfile,"  %s -> setDefaultCursor(getApp () -> getDefaultCursor(%s));\n",Stack -> Name,GetCursor (Cursor));
    }

  // Enabled
  if (Enabled != NULL && strcmp (Enabled,"False") == 0)
    {
      fprintf (cppfile,"  %s -> disable ();\n",Stack -> Name);
    }

  // Event - OnClick
  if (OnClick != NULL)
    {
      // EventFunction
      Push (EventsFunction,OnClick);

      // Message Type
      Push (MessageType,"SEL_COMMAND");

      // MessageID
      char MsgID [512];
      sprintf (MsgID,"ID_%s",Stack -> Name);
      Push (MessageID,MsgID);
    }

  // one empty line
  fprintf (cppfile,"\n");

  // clear
  delete [] Caption;
  delete [] FontColor;
  delete [] Color;
  delete [] Hint;
  delete [] ShowHint;
  delete [] Alignment;
  delete [] FontCharset;
  delete [] FontName;
  delete [] Cursor;
  delete [] Enabled;
  delete [] Checked;
  delete [] OnClick;
}