//---------------------------------------------------------------------------
// TForm Class - default, first class in DFM file
//---------------------------------------------------------------------------

/*
Supported properties:

  BorderStyle
  BorderIcons
  Left
  Top
  ClientHeight
  Height
  ClientWidth
  Width
  Caption
  Color
  Font.Charset
  Font.Color
  Font.Height
  Font.Name
  Font.Style
  Cursor
  Icon.Data

Supported events:

  OnMouseDown
  OnMouseMove
  OnMouseUp
  OnClick
  OnKeyDown
  OnKeyPress
  OnKeyUp
  OnClose
  OnPaint
  OnResize
*/

void DFMParser::TFormClass ()
{
  bool Error;
  char *ObjectName = ReadString (Error);
  SkipChar (':');
  char *ObjectType = ReadString (Error);
  delete [] ObjectType;

  if (ClassGenerate == MAIN_WINDOW)
    {
      // *.cpp file
      fprintf (cppfile,"#include <fx.h>\n");
      fprintf (cppfile,"#include \"test.h\"\n");
      fprintf (cppfile,"\n");
      fprintf (cppfile,"%s::%s(FXApp* app):\n",ObjectName,ObjectName);

      // *.h file
      fprintf (hfile,"#ifndef __Test_H__\n");
      fprintf (hfile,"#define __Test_H__\n");
      fprintf (hfile,"\n");
      fprintf (hfile,"class %s: public FXMainWindow\n",ObjectName);
      fprintf (hfile,"{\n");
      fprintf (hfile,"  FXDECLARE(%s)\n",ObjectName);
      fprintf (hfile,"\n");
      fprintf (hfile,"// Services\n");
      fprintf (hfile,"public:\n");
      fprintf (hfile,"  %s(FXApp* app);\n",ObjectName);
      fprintf (hfile,"\n");
      fprintf (hfile,"private:\n");
      fprintf (hfile,"  %s () {}\n",ObjectName);
      fprintf (hfile,"\n");
      fprintf (hfile,"// Attributes\n");
      fprintf (hfile,"protected:\n");
    }
  else
    {
      // *.cpp file
      fprintf (cppfile,"#include <fx.h>\n");
      fprintf (cppfile,"#include \"test.h\"\n");
      fprintf (cppfile,"\n");
      fprintf (cppfile,"%s::%s(FXWindow* owner):\n",ObjectName,ObjectName);

      // *.h file
      fprintf (hfile,"#ifndef __Test_H__\n");
      fprintf (hfile,"#define __Test_H__\n");
      fprintf (hfile,"\n");
      fprintf (hfile,"class %s: public FXDialogBox\n",ObjectName);
      fprintf (hfile,"{\n");
      fprintf (hfile,"  FXDECLARE(%s)\n",ObjectName);
      fprintf (hfile,"\n");
      fprintf (hfile,"// Services\n");
      fprintf (hfile,"public:\n");
      fprintf (hfile,"  %s(FXWindow* owner);\n",ObjectName);
      fprintf (hfile,"\n");
      fprintf (hfile,"private:\n");
      fprintf (hfile,"  %s () {}\n",ObjectName);
      fprintf (hfile,"\n");
      fprintf (hfile,"// Attributes\n");
      fprintf (hfile,"protected:\n");
    }

  // push and clear the Object name
  Push (Stack,ObjectName);
  delete [] ObjectName;

  // first object, this ...
  Push (Stack,"this");

  // attributes
  int Left;
  int Top;
  int Height;
  int Width;
  int ClientWidth = 0;
  int ClientHeight = 0;
  char *Color = NULL;
  char *Caption = NULL;
  StackElement *BorderIcons = NULL;
  char *BorderStyle = NULL;
  char *FontColor = NULL;
  char *FontCharset = NULL;
  int FontHeight = 0;
  char *FontName = NULL;
  StackElement *FontStyle = NULL;
  char *Cursor = NULL;
  char *IconData = NULL;
  char *OnMouseDown = NULL;
  char *OnMouseMove = NULL;
  char *OnMouseUp = NULL;
  char *OnClick = NULL;
  char *OnKeyDown = NULL;
  char *OnKeyPress = NULL;
  char *OnKeyUp = NULL;
  char *OnClose = NULL;
  char *OnPaint = NULL;
  char *OnResize = NULL;

  // read the words ...
  char *Name = ReadString (Error);
  while (strcmp (Name,"object") != 0 && strcmp (Name,"end") != 0)
    {
      // BorderStyle
      ReadStringProperty (Name,"BorderStyle",BorderStyle);

      // BorderIcons
      ReadSetProperty (Name,"BorderIcons",BorderIcons);

      // Left
      ReadDigitProperty (Name,"Left",Left);

      // Top
      ReadDigitProperty (Name,"Top",Top);

      // Height
      ReadDigitProperty (Name,"Height",Height);

      // Width
      ReadDigitProperty (Name,"Width",Width);

      // ClientHeight
      ReadDigitProperty (Name,"ClientHeight",ClientHeight);

      // ClientWidth
      ReadDigitProperty (Name,"ClientWidth",ClientWidth);

      // Caption
      ReadQuotationProperty (Name,"Caption",Caption);

      // Background Color
      ReadStringProperty (Name,"Color",Color);

      // Font Charset
      ReadStringProperty (Name,"Font.Charset",FontCharset);

      // Font Color
      ReadStringProperty (Name,"Font.Color",FontColor);

      // Font Height
      ReadDigitProperty (Name,"Font.Height",FontHeight);

      // Font Name
      ReadQuotationProperty (Name,"Font.Name",FontName);

      // Font Style
      ReadSetProperty (Name,"Font.Style",FontStyle);

      // Cursor
      ReadStringProperty (Name,"Cursor",Cursor);

      // Icon.Data
      if (strcmp (Name,"Icon.Data") == 0)
        {
          SkipChar ('=');
          IconData = ReadIcon (Error);
        }

      // OnMouseDown
      ReadStringProperty (Name,"OnMouseDown",OnMouseDown);

      // OnMouseMove
      ReadStringProperty (Name,"OnMouseMove",OnMouseMove);

      // OnMouseUp
      ReadStringProperty (Name,"OnMouseUp",OnMouseUp);

      // OnClick
      ReadStringProperty (Name,"OnClick",OnClick);

      // OnKeyDown
      ReadStringProperty (Name,"OnKeyDown",OnKeyDown);

      // OnKeyPress
      ReadStringProperty (Name,"OnKeyPress",OnKeyPress);

      // OnKeyUp
      ReadStringProperty (Name,"OnKeyUp",OnKeyUp);

      // OnClose
      ReadStringProperty (Name,"OnClose",OnClose);

      // OnPaint
      ReadStringProperty (Name,"OnPaint",OnPaint);

      // OnResize
      ReadStringProperty (Name,"OnResize",OnResize);

      delete [] Name;
      Name = ReadString (Error);
    }
  delete [] Name;

  // generate FXMainWindow parent class
  if (ClassGenerate == MAIN_WINDOW)
    {
      fprintf (cppfile,"  FXMainWindow (app,\"");
      if (Caption != NULL)
        fprintf (cppfile,"%s",Caption);
      fprintf (cppfile,"\",NULL,NULL,");
    }
  else
    // generate FXDialogBox parent class
    {
      fprintf (cppfile,"  FXDialogBox (owner,\"");
      if (Caption != NULL)
        fprintf (cppfile,"%s",Caption);
      fprintf (cppfile,"\",");
    }

  // bsSizeable or bsSizeToolWin - default
  if (BorderStyle == NULL || strcmp (BorderStyle,"bsSizeable") == 0 || strcmp (BorderStyle,"bsSizeToolWin") == 0)
    {
      fprintf (cppfile,"DECOR_TITLE | DECOR_BORDER | DECOR_RESIZE");
      bool biSystemMenu = false;
      bool biMinimize = false;
      bool biMaximize = false;

      // search
      if (Empty (BorderIcons))
        {
          biSystemMenu = true;
          biMinimize = true;
          biMaximize = true;
        }
      else
      while (!Empty (BorderIcons))
        {
          if (strcmp (BorderIcons -> Name,"biSystemMenu") == 0)
            biSystemMenu = true;
          else
          if (strcmp (BorderIcons -> Name,"biMinimize") == 0)
            biMinimize = true;
          else
          if (strcmp (BorderIcons -> Name,"biMaximize") == 0)
            biMaximize = true;
          Pop (BorderIcons);
        }
      if (biSystemMenu == true)
        fprintf (cppfile," | DECOR_MENU | DECOR_CLOSE");
      if (biMinimize == true && biSystemMenu == true)
        fprintf (cppfile," | DECOR_MINIMIZE");
      if (biMaximize == true && biSystemMenu == true)
        fprintf (cppfile," | DECOR_MAXIMIZE");
    }
  else

  // bsDialog
  if (strcmp (BorderStyle,"bsDialog") == 0)
    {
      fprintf (cppfile,"DECOR_TITLE");
      bool biSystemMenu = false;

      // search for biSystemMenu
      while (!Empty (BorderIcons))
        {
          if (strcmp (BorderIcons -> Name,"biSystemMenu") == 0)
            biSystemMenu = true;
          Pop (BorderIcons);
        }
      if (biSystemMenu == true)
        fprintf (cppfile," | DECOR_CLOSE | DECOR_MENU");
    }
  else

  // bsSingle or bsToolWindow
  if (strcmp (BorderStyle,"bsSingle") == 0 || strcmp (BorderStyle,"bsToolWindow") == 0)
    {
      fprintf (cppfile,"DECOR_TITLE | DECOR_BORDER");
      bool biSystemMenu = false;
      bool biMinimize = false;
      bool biMaximize = false;

      // search
      while (!Empty (BorderIcons))
        {
          if (strcmp (BorderIcons -> Name,"biSystemMenu") == 0)
            biSystemMenu = true;
          else
          if (strcmp (BorderIcons -> Name,"biMinimize") == 0)
            biMinimize = true;
          else
          if (strcmp (BorderIcons -> Name,"biMaximize") == 0)
            biMaximize = true;
          Pop (BorderIcons);
        }
      if (biSystemMenu == true)
        fprintf (cppfile," | DECOR_MENU | DECOR_CLOSE");
      if (biMinimize == true && biSystemMenu == true)
        fprintf (cppfile," | DECOR_MINIMIZE");
      if (biMaximize == true && biSystemMenu == true)
        fprintf (cppfile," | DECOR_MAXIMIZE");
    }
  else

  // bsNone
  if (strcmp (BorderStyle,"bsNone") == 0)
    {
      fprintf (cppfile,"DECOR_NONE");
      // clear stack
      while (!Empty (BorderIcons))
        Pop (BorderIcons);
    }

  // calculate Width
  if (ClientWidth > 0)
    Width = ClientWidth;
  else
    Width -= 8;

  // calculate Height
  if (ClientHeight > 0)
    Height = ClientHeight;
  else
    Height -=27;

  // Left, Top, Width, Height
  fprintf (cppfile,",%i,%i,%i,%i,0,0,0,0,0,0)\n",Left,Top,Width,Height);
  fprintf (cppfile,"{\n");

  // Color
  if (Color != NULL && strlen (GetColor (Color)) > 0)
    {
      fprintf (cppfile,"  setBackColor (%s);\n",GetColor (Color));
    }

  // Font
  if (FontHeight != 0 && FontName != NULL)
    {
      fprintf (cppfile,"  getApp () -> setNormalFont (new FXFont (getApp(),\"%s\",%i",FontName,(int)((-FontHeight * 72) / 96));

      bool Bold = false;
      bool Italic = false;
      while (!Empty (FontStyle))
        {
          if (strcmp (FontStyle -> Name,"fsBold") == 0)
            Bold = true;
          else
          if (strcmp (FontStyle -> Name,"fsItalic") == 0)
            Italic = true;
          Pop (FontStyle);
        }

      // Bold
      if (Bold == true)
        fprintf (cppfile,",FONTWEIGHT_BOLD");
      else
        fprintf (cppfile,",FONTWEIGHT_NORMAL");

      // Italic
      if (Italic == true)
        fprintf (cppfile,",FONTSLANT_ITALIC");
      else
        fprintf (cppfile,",FONTSLANT_REGULAR");

      // Font.Charset
      if (FontCharset != NULL)
        fprintf (cppfile,",%s));\n",GetFontCharset (FontCharset));
      else
        fprintf (cppfile,"));\n");
    }


  // Cursor

  if (Cursor != NULL && strlen (GetCursor (Cursor)) > 0)
    fprintf (cppfile,"  setDefaultCursor(getApp () -> getDefaultCursor(%s));\n",GetCursor (Cursor));


  // Icon

  if (IconData != NULL)

    {

      // next Icon file...

      IconCount++;

      char name [30];


      // write data

      sprintf (name,"icon%i",IconCount);

      fprintf (cppfile,"  const unsigned char %s []={\n",name);

      fprintf (cppfile,"  ");

      int x = 0;

      for (unsigned int i = 0; i < strlen (IconData); i += 2)

        {

          fprintf (cppfile,"0x%c%c",IconData [i],IconData [i+1]);

          if (i < strlen (IconData) - 2)

            fprintf (cppfile,",");

          else

            fprintf (cppfile,"};\n");

          if (++x == 16)

            {

              fprintf (cppfile,"\n  ");

              x = 0;

            }

        }

      fprintf (cppfile,"  setIcon (new FXICOIcon(getApp(),%s));\n",name);

      fprintf (cppfile,"\n");


      // save the ico file

      if (SaveICO == true)

        {

          strcat (name,".ico");

          SaveIconGlyphFile (name,IconData);

        }

    }


  // Event - OnMouseDown, it's simulated by SEL_LEFTBUTTONPRESS,

  // SEL_MIDDLEBUTTONPRESS and SEL_RIGHTBUTTONPRESS

  if (OnMouseDown != NULL)

    {
      // Event functions
      Push (EventsFunction,OnMouseDown);
      Push (EventsFunction,OnMouseDown);
      Push (EventsFunction,OnMouseDown);

      // Message types
      Push (MessageType,"SEL_LEFTBUTTONPRESS");
      Push (MessageType,"SEL_MIDDLEBUTTONPRESS");
      Push (MessageType,"SEL_RIGHTBUTTONPRESS");

      // MessageIDs
      Push (MessageID,"FXWindow::ID_NONE");
      Push (MessageID,"FXWindow::ID_NONE");
      Push (MessageID,"FXWindow::ID_NONE");
    }


  // Event - OnMouseMove

  if (OnMouseMove != NULL)

    {

      // Event function

      Push (EventsFunction,OnMouseMove);

      // Message type
      Push (MessageType,"SEL_MOTION");

      // MessageID
      Push (MessageID,"FXWindow::ID_NONE");
    }


  // Event - OnMouseUp, it's simulated by SEL_LEFTBUTTONRELEASE,
  // SEL_MIDDLEBUTTONRELEASE and SEL_RIGHTBUTTONRELEASE

  if (OnMouseUp != NULL)

    {
      // Event functions
      Push (EventsFunction,OnMouseUp);
      Push (EventsFunction,OnMouseUp);
      Push (EventsFunction,OnMouseUp);

      // Message types
      Push (MessageType,"SEL_LEFTBUTTONRELEASE");
      Push (MessageType,"SEL_MIDDLEBUTTONRELEASE");
      Push (MessageType,"SEL_RIGHTBUTTONRELEASE");

      // MessageIDs
      Push (MessageID,"FXWindow::ID_NONE");
      Push (MessageID,"FXWindow::ID_NONE");
      Push (MessageID,"FXWindow::ID_NONE");
    }

  // OnClick
  if (OnClick != NULL)
    {

      // Event function

      Push (EventsFunction,OnClick);

      // Message type
      Push (MessageType,"SEL_LEFTBUTTONPRESS");

      // MessageID
      Push (MessageID,"FXWindow::ID_NONE");
    }


  // OnKeyDown
  if (OnKeyDown != NULL)
    {

      // Event function

      Push (EventsFunction,OnKeyDown);

      // Message type
      Push (MessageType,"SEL_KEYPRESS");

      // MessageID
      Push (MessageID,"FXWindow::ID_NONE");
    }


  // OnKeyPress
  if (OnKeyPress != NULL)
    {

      // Event function

      Push (EventsFunction,OnKeyPress);

      // Message type
      Push (MessageType,"SEL_KEYPRESS");

      // MessageID
      Push (MessageID,"FXWindow::ID_NONE");
    }


  // OnKeyUp
  if (OnKeyUp != NULL)
    {

      // Event function

      Push (EventsFunction,OnKeyUp);

      // Message type
      Push (MessageType,"SEL_KEYRELEASE");

      // MessageID
      Push (MessageID,"FXWindow::ID_NONE");
    }


  // OnClose
  if (OnClose != NULL)
    {

      // Event function

      Push (EventsFunction,OnClose);

      // Message type
      Push (MessageType,"SEL_CLOSE");

      // MessageID
      Push (MessageID,"FXWindow::ID_NONE");
    }


  // OnPaint
  if (OnPaint != NULL)
    {

      // Event function

      Push (EventsFunction,OnPaint);

      // Message type
      Push (MessageType,"SEL_PAINT");

      // MessageID
      Push (MessageID,"FXWindow::ID_NONE");
    }


  // OnResize
  if (OnResize != NULL)
    {

      // Event function

      Push (EventsFunction,OnResize);

      // Message type
      Push (MessageType,"SEL_CONFIGURE");

      // MessageID
      Push (MessageID,"FXWindow::ID_NONE");
    }

  // clear
  delete [] Color;
  delete [] Caption;
  delete [] BorderStyle;
  delete [] FontColor;
  delete [] FontCharset;
  delete [] FontName;
  delete [] Cursor;
  delete [] IconData;
  delete [] OnMouseDown;
  delete [] OnMouseMove;
  delete [] OnMouseUp;
  delete [] OnClick;
  delete [] OnKeyDown;
  delete [] OnKeyPress;
  delete [] OnKeyUp;
  delete [] OnClose;
  delete [] OnPaint;
  delete [] OnResize;
}

