//---------------------------------------------------------------------------
// TImage Class
//---------------------------------------------------------------------------

/*
Supported properties:

  Left
  Top
  Width
  Height
  Layout
  Align
  Cursor
  Enabled
  Proportional

Supported events:
  OnClick
*/

void DFMParser::TImageClass ()
{
  // properties
  int  Left;
  int  Top;
  int  Width;
  int  Height;
  char *Layout = NULL;
  char *Align = NULL;
  char *Cursor = NULL;
  char *Enabled = NULL;
  char *PictureData = NULL;
  char *Stretch = NULL;
  char *Proportional = NULL;

  // Events
  char *OnClick = NULL;

  // for detect image type
  enum ImageType { None, BMP, JPEG, ICO, EMF };
  ImageType Type = None;


  // read words...
  bool Error;
  char *Name = ReadString (Error);
  while (strcmp (Name,"object") != 0 && strcmp (Name,"end") != 0)
    {
      // Left
      if (strcmp (Name,"Left") == 0)
        {
          SkipChar ('=');
          Left = ReadDigit (Error);
        }
      else

      // Top
      if (strcmp (Name,"Top") == 0)
        {
          SkipChar ('=');
          Top = ReadDigit (Error);
        }
      else

      // Height
      if (strcmp (Name,"Height") == 0)
        {
          SkipChar ('=');
          Height = ReadDigit (Error);
        }
      else

      // Width
      if (strcmp (Name,"Width") == 0)
        {
          SkipChar ('=');
          Width = ReadDigit (Error);
        }
      else

      // Layout:
      if (strcmp (Name,"Layout") == 0)
        {
          SkipChar ('=');
          Layout = ReadString (Error);
        }
      else

      // Align
      if (strcmp (Name,"Align") == 0)
        {
          SkipChar ('=');
          Align = ReadString (Error);
        }
      else

      // Cursor
      if (strcmp (Name,"Cursor") == 0)
        {
          SkipChar ('=');
          Cursor = ReadString (Error);
        }
      else

      // Enabled
      if (strcmp (Name,"Enabled") == 0)
        {
          SkipChar ('=');
          Enabled = ReadString (Error);
        }
      else

      // OnClick
      if (strcmp (Name,"OnClick") == 0)
        {
          SkipChar ('=');
          OnClick = ReadString (Error);
        }
      else

      // Picture.Data
      if (strcmp (Name,"Picture.Data") == 0)
        {
          SkipChar ('=');
          PictureData = ReadIcon (Error);
        }

      else

      // Stretch
      if (strcmp (Name,"Stretch") == 0)
        {
          SkipChar ('=');
          Stretch = ReadString (Error);
        }
      else

      // Proportional
      if (strcmp (Name,"Proportional") == 0)
        {
          SkipChar ('=');
          Proportional = ReadString (Error);
        }

      // clear old and read new word ...
      delete [] Name;
      Name = ReadString (Error);
    }
  delete [] Name;

  // *.h file
  fprintf (hfile,"   FXImageView *%s;\n",Stack -> Name);

  // *.cpp file

  // Picture
  if (PictureData != NULL)

    {

      // next Glyph file...

      GlyphCount++;

      char name [30];

      sprintf (name,"glyph%i",GlyphCount);


      // detect image type


      // BMP - TBitmap

      char TBitmap [] = {"544269746D6170"};

      Type = BMP;

      int Offset = 24;

      for (unsigned int i = 0; i < strlen (TBitmap); i++)

        {

          if (PictureData [i + 2] != TBitmap [i])

            Type = None;

        }


      // JPEG - TJPEGImager

      if (Type == None)

        {

          Offset = 30;

          Type = JPEG;

          char TJPEGImager [] = {"544A504547496D61676572"};

          for (unsigned int i = 0; i < strlen (TJPEGImager); i++)

            {

              if (PictureData [i + 2] != TJPEGImager [i])

                Type = None;

            }

        }


      // ICO - TIcon

      if (Type == None)

        {

          Offset = 12;

          Type = ICO;

          char TIcon [] = {"5449636F6E"};

          for (unsigned int i = 0; i < strlen (TIcon); i++)

            {

              if (PictureData [i + 2] != TIcon [i])

                Type = None;

            }

        }


      // WMF,EMF - TMetafile

      if (Type == None)

        {

          Offset = 28;

          Type = EMF;

          char TMetafile [] = {"544D65746166696C65"};

          for (unsigned int i = 0; i < strlen (TMetafile); i++)

            {

              if (PictureData [i + 2] != TMetafile [i])

                Type = None;

            }

        }


      // WNF/EMF file

      if (Type == EMF)

        {

          fprintf (cppfile,"  // FOX library don't support WMF/EMF file.\n");

          fprintf (cppfile,"  // Please convert it for other format.\n");

        }

      else

        {

          // write data

          fprintf (cppfile,"  const unsigned char %s []={\n",name);

          fprintf (cppfile,"  ");

          int x = 0;

          for (unsigned int i = Offset; i < strlen (PictureData); i += 2)

            {

              fprintf (cppfile,"0x%c%c",PictureData [i],PictureData [i+1]);

              if (i < strlen (PictureData) - 2)

                fprintf (cppfile,",");

              else

                fprintf (cppfile,"};\n");

              if (++x == 16)

                {

                  fprintf (cppfile,"\n  ");

                  x = 0;

                }

            }


          fprintf (cppfile,"\n");

        }


      // save the picture file

      switch (Type)

        {

          case BMP: strcat (name,".bmp"); break;

          case JPEG: strcat (name,".jpg"); break;

          case ICO: strcat (name,".ico"); break;

          case EMF: strcat (name,".emf"); break;

        }

      if (SavePicture == true)

        {

          SaveIconGlyphFile (name,&PictureData[Offset]);

        }

    }

  fprintf (cppfile,"  %s = new  FXImageView (%s,",Stack -> Name,Stack -> Next -> Name);

  // create image
  switch (Type)
    {
      case BMP:
        char name [30];
        sprintf (name,"glyph%i",GlyphCount);

        fprintf (cppfile,"new FXBMPImage (getApp(),%s),",name);
        break;
      case ICO:
        sprintf (name,"glyph%i",GlyphCount);
        fprintf (cppfile,"new FXICOImage (getApp(),%s),",name);
        break;
      case JPEG:
        sprintf (name,"glyph%i",GlyphCount);
        fprintf (cppfile,"new FXJPEGImage (getApp(),%s),",name);
        break;
      default:
        fprintf (cppfile,"NULL,");
    }

  // any event
  if (OnClick != NULL)
    {
      fprintf (cppfile,"this,ID_%s,",Stack -> Name,OnClick);
    }
  else
    fprintf (cppfile,"NULL,0,");

  // Align
  fprintf (cppfile,"HSCROLLER_NEVER | VSCROLLER_NEVER | %s",GetAlign (Align));

  // Left, Top, Width, Height
  fprintf (cppfile,",%i,%i,%i,%i);\n",Left,Top,Width,Height);

  // Cursor
  if (Cursor != NULL && strlen (GetCursor (Cursor)) > 0)
    {
      fprintf (cppfile,"  %s -> setDefaultCursor(getApp () -> getDefaultCursor(%s));\n",Stack -> Name,GetCursor (Cursor));
    }

  // Enabled
  if (Enabled != NULL && strcmp (Enabled,"False") == 0)
    {
      fprintf (cppfile,"  %s -> disable ();\n",Stack -> Name);
    }

  // Stretch, (ignored for ICO image)
  if (Type != None && Type != ICO && Stretch != NULL && strcmp (Stretch,"True") == 0)
    {
      // Proportional
      if (Proportional != NULL && strcmp (Proportional,"True") == 0)
        {
          fprintf (cppfile,"  %s -> getImage () -> scale (\n",Stack -> Name);
          fprintf (cppfile,"    (float)%i / (float)%i > (float)%s -> getImage () -> getWidth () / (float)%s -> getImage () -> getHeight () ? ",Width,Height,Stack -> Name,Stack -> Name);
          fprintf (cppfile,"%i * (float)%s -> getImage () -> getWidth () / (float)%s -> getImage () -> getHeight () : %i,\n",Height,Stack -> Name,Stack -> Name,Width);
          fprintf (cppfile,"    (float)%i / (float)%i > (float)%s -> getImage () -> getWidth () / (float)%s -> getImage () -> getHeight () ? ",Width,Height,Stack -> Name,Stack -> Name);
          fprintf (cppfile,"%i : %i * (float)%s -> getImage () -> getHeight () / (float)%s -> getImage () -> getWidth ());\n",Height,Width,Stack -> Name,Stack -> Name);
        }
      else
        fprintf (cppfile,"  %s -> getImage () -> scale (%i,%i);\n",Stack -> Name,Width,Height);
    }

  // Event - OnClick
  if (OnClick != NULL)
    {
      // EventFunction
      Push (EventsFunction,OnClick);

      // Message Type
      Push (MessageType,"SEL_LEFTBUTTONPRESS");

      // MessageID
      char MsgID [512];
      sprintf (MsgID,"ID_%s",Stack -> Name);
      Push (MessageID,MsgID);
    }

  // clear
  delete [] Layout;
  delete [] Align;
  delete [] Cursor;
  delete [] Enabled;
  delete [] PictureData;
  delete [] Stretch;
  delete [] Proportional;
  delete [] OnClick;
}
