//---------------------------------------------------------------------------
// TMemo Class
//---------------------------------------------------------------------------

void DFMParser::TMemoClass ()
{
  // attributes
  int  Left;
  int  Top;
  int  Width;
  int  Height;
  char *Text = NULL;
  char *Hint = NULL;
  char *Enabled = NULL;
  char *FontColor = NULL;
  char *FontCharset = NULL;
  int FontHeight = 0;
  char *FontName = NULL;
  StackElement *FontStyle = NULL;
  char *Cursor = NULL;
  char *BorderStyle = NULL;
  char *Color = NULL;
  char *Align = NULL;
  StackElement *LinesStrings = NULL;
  char *Alignment = NULL;

  // read words...
  bool Error;
  char *Name = ReadString (Error);
  while (strcmp (Name,"object") != 0 && strcmp (Name,"end") != 0)
    {
      // Left
      if (strcmp (Name,"Left") == 0)
        {
          SkipChar ('=');
          Left = ReadDigit (Error);
        }
      else

      // Top
      if (strcmp (Name,"Top") == 0)
        {
          SkipChar ('=');
          Top = ReadDigit (Error);
        }
      else

      // Height
      if (strcmp (Name,"Height") == 0)
        {
          SkipChar ('=');
          Height = ReadDigit (Error);
        }
      else

      // Width
      if (strcmp (Name,"Width") == 0)
        {
          SkipChar ('=');
          Width = ReadDigit (Error);
        }
      else

      // Text
      if (strcmp (Name,"Text") == 0)
        {
          SkipChar ('=');
          Text = ReadQuotation (Error);
        }
      else

      // Hint
      if (strcmp (Name,"Hint") == 0)
        {
          SkipChar ('=');
          Hint = ReadQuotation (Error);
        }
      else

      // Enabled
      if (strcmp (Name,"Enabled") == 0)
        {
          SkipChar ('=');
          Enabled = ReadString (Error);
        }
      else

      // Font Charset
      if (strcmp (Name,"Font.Charset") == 0)
        {
          SkipChar ('=');
          FontCharset = ReadString (Error);
        }
      else

      // Font Height
      if (strcmp (Name,"Font.Height") == 0)
        {
          SkipChar ('=');
          FontHeight = ReadDigit (Error);
        }
      else

      // Font Name
      if (strcmp (Name,"Font.Name") == 0)
        {
          SkipChar ('=');
          FontName = ReadQuotation (Error);
        }
      else

      // Font Color
      if (strcmp (Name,"Font.Color") == 0)
        {
          SkipChar ('=');
          FontColor = ReadString (Error);
        }
      else
      // Font Style
      if (strcmp (Name,"Font.Style") == 0)
        {
          SkipChar ('=');
          SkipChar ('[');
          char *Style = NULL;
          while (TextBuf [Pos] != ']')
            {
              Style = ReadString (Error);
              Push (FontStyle,Style);
              delete [] Style;
            }
        }
      else

      // Cursor
      if (strcmp (Name,"Cursor") == 0)
        {
          SkipChar ('=');
          Cursor = ReadString (Error);
        }
      else

      // BorderStyle
      if (strcmp (Name,"BorderStyle") == 0)
        {
          SkipChar ('=');
          BorderStyle = ReadString (Error);
        }
      else

      // Color
      if (strcmp (Name,"Color") == 0)
        {
          SkipChar ('=');
          Color = ReadString (Error);
        }
      else

      // Align
      if (strcmp (Name,"Align") == 0)
        {
          SkipChar ('=');
          Align = ReadString (Error);
        }
      else

      // Alignment
      if (strcmp (Name,"Alignment") == 0)
        {
          SkipChar ('=');
          Alignment = ReadString (Error);
        }
      else

      // Lines.Strings
      if (strcmp (Name,"Lines.Strings") == 0)
        {
          SkipChar ('=');
          SkipChar ('(');
          char *LineString = NULL;
          while (TextBuf [Pos] != ')')
            {
              LineString = ReadQuotation (Error);
              if (Error == false)
                {
                  Push (LinesStrings,LineString);
                  SkipChar ('\'');
                  delete [] LineString;
                }
              else
                Error = false;
            }
        }

      // read new word ...
      delete [] Name;
      Name = ReadString (Error);
    }
  delete [] Name;

  // *.h file
  fprintf (hfile,"  FXText *%s;\n",Stack -> Name);

  // *.cpp file
  fprintf (cppfile,"  %s = new FXText (%s,NULL,0,",Stack -> Name,Stack -> Next -> Name);

  // BorderStyle
  if (BorderStyle == NULL || strcmp (BorderStyle,"bsSingle") == 0)
    {
//      fprintf (cppfile,"FRAME_NORMAL | ");
//      fprintf (cppfile,"FRAME_RAISED | ");
    }

  // Align
  fprintf (cppfile,"%s,",GetAlign (Align));
  fprintf (cppfile,"%i,%i,%i,%i);\n",Left,Top,Width,Height);

  // Text
  if (Text != NULL)
    {
      fprintf (cppfile,"  %s -> setText (\"%s\");\n",Stack -> Name,Text);
    }

  // Hint
  if (Hint != NULL)
    {
      fprintf (cppfile,"  %s -> setTipText (\"%s\");\n",Stack -> Name,Hint);
    }

  // Enabled
  if (Enabled != NULL && strcmp (Enabled,"False") == 0)
    {
      fprintf (cppfile,"  %s -> disable ();\n",Stack -> Name);
    }

  // Font Color

  if (FontColor != NULL && strlen (GetColor (FontColor)) > 0)
    {
      fprintf (cppfile,"  %s -> setTextColor (%s);\n",Stack -> Name,GetColor (FontColor));
    }

  // Font
  if (FontHeight != 0 && FontName != NULL)
    {
      fprintf (cppfile,"  %s -> setFont (new FXFont (getApp(),\"%s\",%i",Stack -> Name,FontName,(int)((-FontHeight * 72) / 96));

      bool Bold = false;
      bool Italic = false;
      while (!Empty (FontStyle))
        {
          if (strcmp (FontStyle -> Name,"fsBold") == 0)
            Bold = true;
          else
          if (strcmp (FontStyle -> Name,"fsItalic") == 0)
            Italic = true;
          Pop (FontStyle);
        }
      if (Bold == true)
        fprintf (cppfile,",FONTWEIGHT_BOLD");
      else
        fprintf (cppfile,",FONTWEIGHT_NORMAL");

      if (Italic == true)
        fprintf (cppfile,",FONTSLANT_ITALIC");
      else
        fprintf (cppfile,",FONTSLANT_REGULAR");

      if (FontCharset != NULL)
        fprintf (cppfile,",%s));\n",GetFontCharset (FontCharset));
      else
        fprintf (cppfile,"));\n");
    }

  // Cursor
  if (Cursor != NULL && strlen (GetCursor (Cursor)) > 0)
    {
      fprintf (cppfile,"  %s -> setDefaultCursor(getApp () -> getDefaultCursor(%s));\n",Stack -> Name,GetCursor (Cursor));
    }
    
  // Color
  if (Color != NULL && strlen (GetColor (Color)) > 0)
    {
      fprintf (cppfile,"  %s -> setBackColor (%s);\n",Stack -> Name,GetColor (Color));
    }

  // Alignment
  if (Alignment != NULL)
   {
     if (strcmp (Alignment,"taCenter") == 0)
       {
  /// Change text style
//   void setTextStyle(FXuint style);
      fprintf (cppfile,"  %s -> setTextStyle (1);\n",Stack -> Name);
       }

   }

//taLeftJustify	Align text to the left side of the control
//taCenter	Center text horizontally in the control.
//taRightJustify	Align text to the right side of the control.

  // reverse Lines.Strings stack
  Reverse (LinesStrings);
/*
  StackElement *ReverseLinesStrings = NULL;
  while (!Empty (LinesStrings))
    {
      Push (&ReverseLinesStrings,LinesStrings -> Name);
      Pop (&LinesStrings);
    }
*/
  // ItemsStrings
//  LinesStrings = ReverseLinesStrings;
  while (!Empty (LinesStrings))
    {
      // *.cpp file
      fprintf (cppfile,"  %s -> appendText (\"%s\\n\",%i);\n",
        Stack -> Name,LinesStrings -> Name,strlen (LinesStrings -> Name) + 1);

      // pop the Item
      Pop (LinesStrings);
    }

  // one empty line
  fprintf (cppfile,"\n");

  // clear
  delete [] Text;
  delete [] FontColor;
  delete [] Hint;
  delete [] FontCharset;
  delete [] FontName;
  delete [] Enabled;
  delete [] Cursor;
  delete [] BorderStyle;
  delete [] Color;
}
