//---------------------------------------------------------------------------
// TPanel Class
//---------------------------------------------------------------------------

/*
Supported properties:

  Left
  Top
  Width
  Height
  Color
  Align
  BevelOuter
  BevelInner
  Cursor
  Caption
  Alignment
  Font.Charset
  Font.Height
  Font.Name
  Font.Style
  Font.Color
*/

void DFMParser::TPanelClass ()
{
  // attributes
  int Left;
  int Top;
  int Width;
  int Height;
  char *Color = NULL;
  char *Align = NULL;
  char *BevelOuter = NULL;
  char *BevelInner = NULL;
  char *Cursor = NULL;
  char *Caption = NULL;
  char *Alignment = NULL;
  char *FontCharset = NULL;
  int FontHeight = 0;
  char *FontName = NULL;
  StackElement *FontStyle = NULL;
  char *FontColor = NULL;

  // read words...
  bool Error;
  char *Name = ReadString (Error);
  while (strcmp (Name,"object") != 0 && strcmp (Name,"end") != 0)
    {
      // Left
      ReadDigitProperty (Name,"Left",Left);

      // Top
      ReadDigitProperty (Name,"Top",Top);

      // Height
      ReadDigitProperty (Name,"Height",Height);

      // Width
      ReadDigitProperty (Name,"Width",Width);

      // Color
      ReadStringProperty (Name,"Color",Color);

      // Align
      ReadStringProperty (Name,"Align",Align);

      // BevelOuter
      ReadStringProperty (Name,"BevelOuter",BevelOuter);

      // BevelInner
      ReadStringProperty (Name,"BevelInner",BevelInner);

      // Cursor
      ReadStringProperty (Name,"Cursor",Cursor);

      // Caption
      ReadQuotationProperty (Name,"Caption",Caption);

      // Alignment
      ReadStringProperty (Name,"Alignment",Alignment);

      // Font Charset
      ReadStringProperty (Name,"Font.Charset",FontCharset);

      // Font Color
      ReadStringProperty (Name,"Font.Color",FontColor);

      // Font Height
      ReadDigitProperty (Name,"Font.Height",FontHeight);

      // Font Name
      ReadQuotationProperty (Name,"Font.Name",FontName);

      // Font Style
      ReadSetProperty (Name,"Font.Style",FontStyle);

      // read new word ...
      delete [] Name;
      Name = ReadString (Error);
    }
  delete [] Name;

  // *.h file
  fprintf (hfile,"  FXGroupBox *%s;\n",Stack -> Name);

  // *.cpp file
  fprintf (cppfile,"  %s = new FXGroupBox (%s,\"\",",Stack -> Name,Stack -> Next -> Name);

  // Align
  fprintf (cppfile,"%s",GetAlign (Align));

  // BecelOuter & BevelInner
  if (BevelOuter == NULL || strcmp (BevelOuter,"bvRaised") == 0 || strcmp (BevelOuter,"bvSpace") == 0)
    {
      if (BevelInner == NULL || strcmp (BevelInner,"bvNone") == 0 ||
          strcmp (BevelInner,"bvRaised") == 0 || strcmp (BevelInner,"bvSpace") == 0)
        fprintf (cppfile," | FRAME_RAISED");
      else

      if (strcmp (BevelInner,"bvLowered") == 0)
        fprintf (cppfile," | FRAME_NORMAL | FRAME_RAISED");
    }
  else

  if (strcmp (BevelOuter,"bvNone") == 0)
    {
      if (BevelInner == NULL || strcmp (BevelInner,"bvNone") == 0)
        fprintf (cppfile," | FRAME_NONE");
      else

      if (strcmp (BevelInner,"bvRaised") == 0 || strcmp (BevelInner,"bvSpace") == 0)
        fprintf (cppfile," | FRAME_RAISED");
      else

      if (strcmp (BevelInner,"bvLowered") == 0)
        fprintf (cppfile," | FRAME_SUNKEN");
    }
  else
    // bvLowered
    {
      if (BevelInner == NULL || strcmp (BevelInner,"bvNone") == 0 || strcmp (BevelInner,"bvLowered") == 0)
        fprintf (cppfile," | FRAME_SUNKEN");
      else

      if (strcmp (BevelInner,"bvRaised") == 0 || strcmp (BevelInner,"bvSpace") == 0)
        fprintf (cppfile," | FRAME_THICK");
    }

  fprintf (cppfile,",%i,%i,%i,%i,0,0,0,0);\n",Left,Top,Width,Height);

  // Color
  if (Color != NULL && strlen (GetColor (Color)) > 0)
    {
      fprintf (cppfile,"  %s -> setBackColor (%s);\n",Stack -> Name,GetColor (Color));
    }

  // Cursor
  if (Cursor != NULL && strlen (GetCursor (Cursor)) > 0)
    {
      fprintf (cppfile,"  %s -> setDefaultCursor(getApp () -> getDefaultCursor(%s));\n",Stack -> Name,GetCursor (Cursor));
    }

  // Caption
  if (Caption != NULL && strlen (Caption) > 0)
    {
      // *.h file
      fprintf (hfile,"  FXLabel *%sLabel;\n",Stack -> Name);

      // *.cpp file
      fprintf (cppfile,"  %sLabel = new FXLabel (%s,",Stack -> Name,Stack -> Name);

      // Caption
      fprintf (cppfile,"\"%s\",NULL,JUSTIFY_CENTER_Y | LAYOUT_FILL_X | LAYOUT_FILL_Y",Caption);

      // Alignment
      if (Alignment == NULL)
        fprintf (cppfile," | JUSTIFY_CENTER_X");
      else
        fprintf (cppfile," | %s",GetAlignment (Alignment));

      fprintf (cppfile,",0,0,0,0,0,0,0,0);\n");

     // Color
     if (Color != NULL && strlen (GetColor (Color)) > 0)
       {
         fprintf (cppfile,"  %sLabel -> setBackColor (%s);\n",Stack -> Name,GetColor (Color));
       }

      // Font
      if (FontHeight != 0 && FontName != NULL)
        {
          fprintf (cppfile,"  %sLabel -> setFont (new FXFont (getApp(),\"%s\",%i",Stack -> Name,FontName,(int)((-FontHeight * 72) / 96));

          bool Bold = false;
          bool Italic = false;
          while (!Empty (FontStyle))
            {
              if (strcmp (FontStyle -> Name,"fsBold") == 0)
                Bold = true;
              else
              if (strcmp (FontStyle -> Name,"fsItalic") == 0)
                Italic = true;
              Pop (FontStyle);
            }

          // Bold
          if (Bold == true)
            fprintf (cppfile,",FONTWEIGHT_BOLD");
          else
            fprintf (cppfile,",FONTWEIGHT_NORMAL");

          // Italic
          if (Italic == true)
            fprintf (cppfile,",FONTSLANT_ITALIC");
          else
            fprintf (cppfile,",FONTSLANT_REGULAR");

          // Charset
          if (FontCharset != NULL)
            fprintf (cppfile,",%s));\n",GetFontCharset (FontCharset));
          else
            fprintf (cppfile,"));\n");
        }

      // Font Color
      if (FontColor != NULL && strlen (GetColor (FontColor)) > 0)
        {
          fprintf (cppfile,"  %sLabel -> setTextColor (%s);\n",Stack -> Name,GetColor (FontColor));
        }

     // Cursor
     if (Cursor != NULL && strlen (GetCursor (Cursor)) > 0)
       {
         fprintf (cppfile,"  %sLabel -> setDefaultCursor(getApp () -> getDefaultCursor(%s));\n",Stack -> Name,GetCursor (Cursor));
       }
    }


  // one empty line
  fprintf (cppfile,"\n");

  // clear
  delete [] Color;
  delete [] Align;
  delete [] BevelOuter;
  delete [] BevelInner;
  delete [] Cursor;
  delete [] Caption;
  delete [] Alignment;
  delete [] FontCharset;
  delete [] FontStyle;
  delete [] FontName;
  delete [] FontColor;
}
