//---------------------------------------------------------------------------
// TRadioGroup Class
//---------------------------------------------------------------------------

/*
Supported properties:

  Left
  Top
  Width
  Height
  Color
  Align
  Cursor
  Caption
  Font.Charset
  Font.Height
  Font.Name
  Font.Style
  Font.Color
  Items.Strings
  ItemIndex
  Enabled
*/


void DFMParser::TRadioGroupClass ()
{
  // attributes
  int Left;
  int Top;
  int Width;
  int Height;
  char *Color = NULL;
  char *Align = NULL;
  char *Cursor = NULL;
  char *Caption = NULL;
  char *FontCharset = NULL;
  int FontHeight = 0;
  char *FontName = NULL;
  StackElement *FontStyle = NULL;
  char *FontColor = NULL;
  StackElement *ItemsStrings = NULL;
  int ItemIndex = -1;
  char *Enabled = NULL;

  // read words...
  bool Error;
  char *Name = ReadString (Error);
  while (strcmp (Name,"object") != 0 && strcmp (Name,"end") != 0)
    {
      // Left
      if (strcmp (Name,"Left") == 0)
        {
          SkipChar ('=');
          Left = ReadDigit (Error);
        }
      else

      // Top
      if (strcmp (Name,"Top") == 0)
        {
          SkipChar ('=');
          Top = ReadDigit (Error);
        }
      else

      // Height
      if (strcmp (Name,"Height") == 0)
        {
          SkipChar ('=');
          Height = ReadDigit (Error);
        }
      else

      // Width
      if (strcmp (Name,"Width") == 0)
        {
          SkipChar ('=');
          Width = ReadDigit (Error);
        }
      else

      // Color
      if (strcmp (Name,"Color") == 0)
        {
          SkipChar ('=');
          Color = ReadString (Error);
        }
      else

      // Align
      if (strcmp (Name,"Align") == 0)
        {
          SkipChar ('=');
          Align = ReadString (Error);
        }

      else

      // Cursor
      if (strcmp (Name,"Cursor") == 0)
        {
          SkipChar ('=');
          Cursor = ReadString (Error);
        }
      else

      // Caption
      if (strcmp (Name,"Caption") == 0)
        {
          SkipChar ('=');
          Caption = ReadQuotation (Error);
        }
      else

      // Font Charset
      if (strcmp (Name,"Font.Charset") == 0)
        {
          SkipChar ('=');
          FontCharset = ReadString (Error);
        }
      else

      // Font Color
      if (strcmp (Name,"Font.Color") == 0)
        {
          SkipChar ('=');
          FontColor = ReadString (Error);
        }
      else

      // Font Height
      if (strcmp (Name,"Font.Height") == 0)
        {
          SkipChar ('=');
          FontHeight = ReadDigit (Error);
        }
      else

      // Font Name
      if (strcmp (Name,"Font.Name") == 0)
        {
          SkipChar ('=');
          FontName = ReadQuotation (Error);
        }
      else
      
      // Font Style
      if (strcmp (Name,"Font.Style") == 0)
        {
          SkipChar ('=');
          SkipChar ('[');
          char *Style = NULL;
          while (TextBuf [Pos] != ']')
            {
              Style = ReadString (Error);
              Push (FontStyle,Style);
              delete [] Style;
            }
        }
      else

      // Items.Strings
      if (strcmp (Name,"Items.Strings") == 0)
        {
          SkipChar ('=');
          SkipChar ('(');
          char *ItemString = NULL;
          while (TextBuf [Pos] != ')')
            {
              ItemString = ReadQuotation (Error);
              Push (ItemsStrings,ItemString);
              SkipChar ('\'');
              delete [] ItemString;
            }
        }
      else

      // ItemIndex
      if (strcmp (Name,"ItemIndex") == 0)
        {
          SkipChar ('=');
          ItemIndex = ReadDigit (Error);
        }
      else

      // Enabled
      if (strcmp (Name,"Enabled") == 0)
        {
          SkipChar ('=');
          Enabled = ReadString (Error);
        }

      // read new word ...
      delete [] Name;
      Name = ReadString (Error);
    }
  delete [] Name;

  // *.h file
  fprintf (hfile,"  FXGroupBox *%s;\n",Stack -> Name);

  // *.cpp file
  fprintf (cppfile,"  %s = new FXGroupBox (%s,\"",Stack -> Name,Stack -> Next -> Name);

  // Caption
  if (Caption != NULL && strlen (Caption) > 0)
    {
      fprintf (cppfile,"%s",Caption);
    }

  // Align and other...
  fprintf (cppfile,"\",%s | FRAME_THICK,%i,%i,%i,%i,0,0,0,0);\n",GetAlign (Align),Left,Top,Width,Height);

  // Color
  if (Color != NULL && strlen (GetColor (Color)) > 0)
    {
      fprintf (cppfile,"  %s -> setBackColor (%s);\n",Stack -> Name,GetColor (Color));
    }

  // Cursor
  if (Cursor != NULL && strlen (GetCursor (Cursor)) > 0)
    {
      fprintf (cppfile,"  %s -> setDefaultCursor(getApp () -> getDefaultCursor(%s));\n",Stack -> Name,GetCursor (Cursor));
    }

  // Font
  if (FontHeight != 0 && FontName != NULL)
    {
      fprintf (cppfile,"  %s -> setFont (new FXFont (getApp(),\"%s\",%i",Stack -> Name,FontName,(int)((-FontHeight * 72) / 96));

      bool Bold = false;
      bool Italic = false;
      while (!Empty (FontStyle))
        {
          if (strcmp (FontStyle -> Name,"fsBold") == 0)
            Bold = true;
          else
          if (strcmp (FontStyle -> Name,"fsItalic") == 0)
            Italic = true;
          Pop (FontStyle);
        }

      // Bold
      if (Bold == true)
        fprintf (cppfile,",FONTWEIGHT_BOLD");
      else
        fprintf (cppfile,",FONTWEIGHT_NORMAL");

      // Italic
      if (Italic == true)
        fprintf (cppfile,",FONTSLANT_ITALIC");
      else
        fprintf (cppfile,",FONTSLANT_REGULAR");

      // Charset
      if (FontCharset != NULL)
        fprintf (cppfile,",%s));\n",GetFontCharset (FontCharset));
      else
        fprintf (cppfile,"));\n");
    }

  // Font Color
  if (FontColor != NULL && strlen (GetColor (FontColor)) > 0)
    {
      fprintf (cppfile,"  %s -> setTextColor (%s);\n",Stack -> Name,GetColor (FontColor));
    }

  // Enabled
  if (Enabled != NULL && strcmp (Enabled,"False") == 0)
    {
      fprintf (cppfile,"  %s -> disable ();\n",Stack -> Name);
    }
    
  // reverse Items.Strings stack
  Reverse (ItemsStrings);

  // ItemsStrings
  int Count = 0;
  while (!Empty (ItemsStrings))
    {
//  FXRadioButton(FXComposite* p,const FXString& text,FXObject* tgt=NULL,FXSelector sel=0,
//                FXuint opts=RADIOBUTTON_NORMAL,FXint x=0,FXint y=0,FXint w=0,FXint h=0,
//                FXint pl=DEFAULT_PAD,FXint pr=DEFAULT_PAD,FXint pt=DEFAULT_PAD,FXint pb=DEFAULT_PAD);

      // *.h file
      fprintf (hfile,"  FXRadioButton *%sRadioButton%i;\n",Stack -> Name,Count);

      // *.cpp file
      fprintf (cppfile,"  %sRadioButton%i = new FXRadioButton (%s,\"%s\",NULL,0,ICON_BEFORE_TEXT);\n",Stack -> Name,Count,Stack -> Name,ItemsStrings -> Name);

      // ItemIndex
      if (ItemIndex == Count)
        {
          fprintf (cppfile,"  %sRadioButton%i -> setCheck(TRUE);\n",Stack -> Name,Count);
        }

      // Enabled
      if (Enabled != NULL && strcmp (Enabled,"False") == 0)
        {
          fprintf (cppfile,"  %sRadioButton%i -> disable ();\n",Stack -> Name,Count);
        }

      // pop the Item
      Pop (ItemsStrings);

      // next Item
      Count++;
    }


  // one empty line
  fprintf (cppfile,"\n");

  // clear
  delete [] Color;
  delete [] Align;
  delete [] Cursor;
  delete [] Caption;
  delete [] FontCharset;
  delete [] FontStyle;
  delete [] FontName;
  delete [] FontColor;
  delete [] Enabled;
}
